/*
 * Copyright 2011 NEHTA
 *
 * Licensed under the NEHTA Open Source (Apache) License; you may not use this
 * file except in compliance with the License. A copy of the License is in the
 * 'license.txt' file, which should be provided with this work.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
package au.gov.nehta.vendorlibrary.common.security;

import au.gov.nehta.common.utils.ArgumentUtils;
import au.gov.nehta.vendorlibrary.common.DOMUtil;
import au.gov.nehta.xsp.CertificateVerificationException;
import au.gov.nehta.xsp.CertificateVerifier;
import au.gov.nehta.xsp.SignatureValidationException;
import au.gov.nehta.xsp.SignedContainerProfileService;
import au.gov.nehta.xsp.XspException;
import au.gov.nehta.xsp.XspFactory;
import au.gov.nehta.xsp.XspVersion;
import org.w3c.dom.Document;

import javax.security.auth.x500.X500PrivateCredential;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

public final class SignedContainerProfileUtil {

  /**
   * CertificateVerifier instance to sign/verify signed data. Validation ignored with an assumption that the certificates
   * under test are valid. <b>MUST INCLUDE VALIDATION CODE FOR PRODUCTION PURPOSE</b>
   */
  public static final CertificateVerifier NULL_CERTIFICATE_VERIFIER = new CertificateVerifier() {
    public void verify(X509Certificate certificate)
      throws CertificateVerificationException, XspException {
      // Do nothing. Implement this method for production code.
    }
  };

  private static final Logger LOG = Logger.getLogger(SignedContainerProfileUtil.class.getName());

  /**
   * Default private constructor for Utility class.
   */
  private SignedContainerProfileUtil() {
  }

  public static Document getSignedPayload(final String sensitivePayload, final X509Certificate siginingCert,
                                          final PrivateKey siginingPrivateKey) throws XspException {
    ArgumentUtils.checkNotNullNorBlank(sensitivePayload, "sensitivePayload");
    ArgumentUtils.checkNotNull(siginingCert, "siginingCert");
    ArgumentUtils.checkNotNull(siginingPrivateKey, "siginingPrivateKey");

    // Get the signing private credentials
    List<X500PrivateCredential> certificateKeyPairs = new ArrayList<X500PrivateCredential>();
    certificateKeyPairs.add(new X500PrivateCredential(siginingCert,
      siginingPrivateKey));

    return getSignedPayload(sensitivePayload, certificateKeyPairs);

  }


  public static Document getSignedPayload(final String sensitivePayload,
                                          final List<X500PrivateCredential> certificateKeyPairs) throws XspException {
    try {
      ArgumentUtils.checkNotNullNorBlank(sensitivePayload, "sensitivePayload");
      ArgumentUtils.checkNotNull(certificateKeyPairs, "certificateKeyPairs");
      if (certificateKeyPairs.size() > 0) {
        Document signedDocument = null;

        Document docToSign = DOMUtil.getDocumentFromXML(sensitivePayload);
        signedDocument = getSignedContainerProfileService().create(docToSign,
          certificateKeyPairs);
        return signedDocument;
      }
    } catch (XspException e) {
      throw new XspException("Failed to sign the document.");
    }
    throw new XspException("Failed to sign the document.");
  }

  /**
   * Returns the extracted data/payload form the provided signedPayload
   * Note: <br>
   * This uses the default private key from the keystore.
   *
   * @param signedDocument
   * @return the payload as {@link Document}
   */
  public static Document getDataFromSignedPayload(
    final Document signedDocument) throws XspException {
    ArgumentUtils.checkNotNull(signedDocument, "signedDocument");
    Document sensitivePayload = null;
    try {
      sensitivePayload = getSignedContainerProfileService().getData(signedDocument);
      ArgumentUtils.checkNotNull(sensitivePayload, "extractedDocument");
      return sensitivePayload;
    } catch (IllegalArgumentException ex) {
      throw new XspException(ex);
    } catch (XspException ex) {
      // TODO: If this exception is caught, it will never be thrown.
    }
    return sensitivePayload;
  }


  /**
   * Returns the extracted data/payload form the provided signedPayload Document
   * Note: <br>
   * This uses the default private key from the keystore.
   *
   * @param signedDocument
   * @return the payload as {@link Document}
   */
  public static boolean verifySignature(
    final Document signedDocument) throws SignatureValidationException {
    boolean signatureVerified = false;
    ArgumentUtils.checkNotNull(signedDocument, "signedDocument");
    Document sensitivePayload = null;
    try {
      getSignedContainerProfileService().check(signedDocument, NULL_CERTIFICATE_VERIFIER);
      signatureVerified = true;
    } catch (XspException e) {
      throw new SignatureValidationException("Failed to verify the signature", e);
    } catch (CertificateVerificationException e) {
      throw new SignatureValidationException("Failed to verify the signature", e);
    }
    return signatureVerified;
  }


  private static SignedContainerProfileService getSignedContainerProfileService() throws XspException {
    return XspFactory.getInstance()
      .getSignedContainerProfileService(XspVersion.V_2010);
  }


}



